@@ -1,4 +1,4 @@
-setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, statsFcn=function(x) quantile(x, probs=c(0.025, 0.050, 0.95, 0.975), na.rm=TRUE), by=c("betaTN", "betaT"), ..., seed=NULL, verbose=FALSE) {
+setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, statsFcn=function(x) quantile(x, probs=c(0.025, 0.050, 0.95, 0.975), na.rm=TRUE), by=c("betaTN", "betaT"), ..., seed=NULL, force=FALSE, verbose=FALSE) {
   # Settings for sanity checks
   tol <- getOption("PSCBS/sanityChecks/tolerance", 0.0005);
 
@@ -16,6 +16,9 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
     seed <- Arguments$getInteger(seed);
   }
 
+  # Argument 'force':
+  force <- Arguments$getLogical(force);
+
   # Argument 'verbose':
   verbose <- Arguments$getVerbose(verbose);
   if (verbose) {
@@ -92,8 +95,8 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
   verbose && cat(verbose, "Already done?");
   verbose && print(verbose, isDone);
 
-  if (all(isDone)) {
-    verbose && cat(verbose, "Already done.");
+  if (!force && all(isDone)) {
+    verbose && cat(verbose, "Already done. Skipping.");
     verbose && exit(verbose);
     return(fit);
   }
@@ -169,7 +172,7 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
   data$rho <- rho;
 
   rm(betaT); # Not needed anymore
-  
+
 
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
@@ -226,11 +229,21 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
       idxsAll <- integer(0);
     }
 
+    verbose && str(verbose, idxsAll);
+    verbose && print(verbose, hpaste(idxsAll));
+    verbose && str(verbose, idxsCT);
+    verbose && print(verbose, hpaste(idxsCT));
+
     # Keep only loci with finite TCNs
     idxsAll <- intersect(idxsAll, idxsCT);
+    verbose && str(verbose, idxsAll);
+    verbose && print(verbose, hpaste(idxsAll));
 
     # Sanity check
-    stopifnot(length(idxsAll) == nbrOfTCNs);
+    if (length(idxsAll) != nbrOfTCNs) {
+      verbose && str(verbose, setdiff(idxsCT, idxsAll));
+      throw("INTERNAL ERROR: length(idxsAll) != nbrOfTCNs: ", length(idxsAll), " != ", nbrOfTCNs);
+    }
 
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -302,6 +315,9 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
     idxsTCN <- sort(unique(c(idxsHet, idxsHom, idxsNonSNP)));
     stopifnot(length(idxsTCN) == nbrOfTCNs);
 
+    verbose && cat(verbose, "Loci to resample for TCN:");
+    verbose && str(verbose, idxsTCN);
+
     verbose && exit(verbose);
 
 
@@ -356,10 +372,14 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
 
         # Calculate bootstrap mean
         M[jj,bb,"dh"] <- mean(rhoBB, na.rm=TRUE);
-      } # if (nbrOfHets > 0)
+      } else {
+        idxsHetNonDH <- idxsDH;
+      } # if (shouldHaveDHs > 0)
   
       # (2) Bootstrap TCNs
       if (nbrOfTCNs > 0) {
+### str(list(idxsHetNonDH=idxsHetNonDH, idxsHom=idxsHom, idxsNonSNP=idxsNonSNP));
+
         # (a) Resample non-DH hets SNPs
         idxsHetNonDHBB <- resample(idxsHetNonDH, size=length(idxsHetNonDH), replace=TRUE);
         idxsHetBB <- c(idxsDHBB, idxsHetNonDHBB);
@@ -470,7 +490,7 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
   # Drop previously estimated values
   dups <- duplicated(colnames(segs), fromLast=TRUE);
   if (any(dups)) {
-    stats <- stats[,!dups, drop=FALSE];
+    segs <- segs[,!dups, drop=FALSE];
   }
 
 
@@ -501,12 +521,14 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
           stopifnot(is.matrix(Skk));
   
           range <- Skk[,c(1,ncol(Skk)),drop=FALSE];
-          verbose && str(verbose, range);
     
           # Segmentation means
           key <- sprintf("%sMean", field);
           segMean <- segs[[key]];
-          verbose && str(verbose, segMean);
+
+          # Segmentation mean statistics
+          verbose && printf(verbose, "mean: %g [%g,%g]\n", segMean, range[1], range[2]);
+          verbose && printf(verbose, "mean: %g, range: [%g,%g]\n", segMean, range[1], range[2]);
   
           # Segmentation counts
           cfield <- sprintf("%sNbrOfLoci", ifelse(field == "tcn", "tcn", "dh"));
@@ -526,6 +548,7 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
         } # for (kk ...)
       }, error = function(ex) {
         # If an error, display the data, then throw the exception
+        verbose && cat(verbose, "Tolerance (option 'PSCBS/sanityChecks/tolerance'): ", tol);
         verbose && print(verbose, segs);
         throw(ex);
       })
@@ -550,6 +573,16 @@ setMethodS3("bootstrapTCNandDHByRegion", "PairedPSCBS", function(fit, B=1000, st
 
 ##############################################################################
 # HISTORY
+# 2012-02-26
+# o BUG FIX: bootstrapTCNandDHByRegion() for PairedPSCBS would resample
+#   from a subset of the intended TCNs, iff the DH mean was non-finite
+#   while there were still heterozygous SNPs.  This introduced a bias in
+#   the estimates, which was neglectable for large segments, but for very
+#   small segments (a few loci) it could be relatively large.
+# 2012-02-24
+# o BUG FIX: bootstrapTCNandDHByRegion(..., force=TRUE) for PairedPSCBS
+#   would give an error iff previous bootstrap estimates already existed.
+# o Added argument 'force' to bootstrapTCNandDHByRegion().
 # 2011-11-26
 # o Now bootstrapTCNandDHByRegion() for PairedPSCBS preserves NAs for DH
 #   and (C1,C2) quantiles, if the DH mean level is NA, which can happen
