@@ -37,18 +37,6 @@ setConstructorS3("PairedPSCBS", function(fit=list(), ...) {
 })
 
 
-setMethodS3("getSegmentSizes", "PairedPSCBS", function(fit, by=c("length", "count"), ...) {
-  by <- match.arg(by);
-
-  data <- getSegments(fit, ...);
-  if (by == "length") {
-    res <- data[["tcnEnd"]]-data[["tcnStart"]]+1L;
-  } else if (by == "count") {
-    res <- data[["tcnNbrOfLoci"]];
-  }
-  res;
-})
-
 
 setMethodS3("updateMeans", "PairedPSCBS", function(fit, from=c("loci", "segments"), adjustFor=NULL, ..., verbose=FALSE) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -119,9 +107,6 @@ setMethodS3("updateMeans", "PairedPSCBS", function(fit, from=c("loci", "segments
     x <- data$x;
     CT <- data$CT;
     rho <- data$rho;
-    muN <- data$muN;
-    isSnp <- !is.na(muN);
-    isHet <- isSnp & (muN == 1/2);
 
     isSplitter <- isSegmentSplitter(fit);
     for (ss in seq(length=nbrOfSegments)[!isSplitter]) {
@@ -322,6 +307,9 @@ setMethodS3("resegment", "PairedPSCBS", function(fit, ..., verbose=FALSE) {
 
 ##############################################################################
 # HISTORY
+# 2012-04-21
+# o CLEANUP: Removed unused objects in updateMeans().
+# o CLEANUP: Moved getSegmentSizes() from PairedPSCBS to PSCBS.
 # 2011-11-21
 # o BUG FIX: resegment() was trying to call segmentByCBS() instead
 #   of segmentByPairedPSCBS().
