@@ -114,7 +114,7 @@ setMethodS3("weightedQuantile", "default", function(x, w, probs=c(0, 0.25, 0.5,
   } else {
     throw("Cannot estimate weighted quantiles: Argument 'method' is unknown: ", method);
   }
-}) # weightedMedian()   
+}) # weightedQuantile()
 
 
 ############################################################################
